/**
 * Iterate deep nested keys
 *
 * @param source: object
 * @param action: Function If property is not object, do Action
 */
function iterate(source, action) {
  Object.keys(source).forEach(function (key) {
    var property = Reflect.get(source, key);

    if (isObject(property)) {
      iterate(property, action);
      return;
    }

    action(source, key);
  });
}
/**
 * Check parameter is nullish
 *
 * @param value
 * @returns boolean If return true value is nullish
 */


function isNullish(value) {
  return value === undefined || value === null;
}
/**
 * Check parameter is Object
 *
 * @param value
 * @returns boolean If return true value is Object
 */


function isObject(value) {
  return value === Object(value);
}

// @ts-ignore
/**
 * Do remove nullish
 *
 * @param source Object has nullish or not
 * @param key source's key
 */

function doRemove(source, key) {
  if (!Reflect.has(source, key)) return;
  if (isNullish(Reflect.get(source, key))) Reflect.deleteProperty(source, key);
}
/**
 * Remove Nullish value.
 *
 * nullish : undefined, null
 *
 * @returns Object
 */


function removeNullish(source) {
  var result = Object.assign({}, source);
  iterate(result, doRemove);
  return result;
}

/**
 * FindKey by key string
 *
 * can use dot notation for deep nested value
 *
 * eg:
 *
 * ```ts
 *import NOM from 'nested-object-mutator'
 *
 * const source = {
 *  a: 1,
 *  b: {
 *    c: 2
 *  }
 *}
 *
 * const result = NOM.getByKey(source, "a.b.c", ".")
 *
 * The result is `2`. return null if not exists.
 *```
 *
 * @param source {object}
 * @param target {string} target key.
 * @param delimiter {string} default delimiter is `.`
 * @returns true if exist
 */

function getByKey(source, target, delimiter) {
  if (delimiter === void 0) {
    delimiter = ".";
  }

  var keys = target.split(delimiter).filter(function (k) {
    return k.trim() !== "";
  });

  if (keys.length === 0) {
    return null;
  }

  var cache = source;

  for (var index = 0; index < keys.length; index++) {
    var currentKey = keys[index];
    var isKeyExists = false;

    if (isObject(cache)) {
      isKeyExists = Reflect.has(cache, currentKey);
    }

    if (!isKeyExists) {
      cache = null;
      continue;
    }

    var currentValue = Reflect.get(cache, currentKey);
    cache = currentValue;
  }

  return cache;
}

var index = {
  removeNullish: removeNullish,
  getByKey: getByKey
};

export default index;
//# sourceMappingURL=nested-object-mutator.esm.js.map
